from node import Node
from queue import Queue, PriorityQueue


def bfs(problem):
    
    frontier = Queue()
    visited = set([])
    
    frontier.put(Node(problem.start))
    
    while not frontier.empty():
        
        curr_node = frontier.get()
        
        if problem.is_goal(curr_node.state):
            return curr_node.get_path()
        
        visited.add(curr_node.state)
        
        for next_node in curr_node.expand(problem):
            if next_node.state not in visited:
                frontier.put(next_node)
    
    return None


def dfs(problem):
    
    frontier = [Node(problem.start)]
    visited = set([])
    
    while len(frontier) > 0:
        
        curr_node = frontier.pop()
        
        if problem.is_goal(curr_node.state):
            return curr_node.get_path()
        
        visited.add(curr_node.state)
        
        for next_node in curr_node.expand(problem):
            if next_node.state not in visited:
                frontier.append(next_node)
    
    return None


# uniform cost search
def ucs(problem):
    
    frontier = PriorityQueue()
    visited = set([])
    
    frontier.put(Node(problem.start))
    
    while not frontier.empty():
        
        curr_node = frontier.get()
        
        if problem.is_goal(curr_node.state):
            return curr_node.get_path()
        
        visited.add(curr_node.state)
        
        for next_node in curr_node.expand(problem):
            if next_node.state not in visited:
                frontier.put(next_node)
    
    return None